#!/usr/bin/env python

import sys
import os
import json
import argparse
import logging
import copy
import tooldistillator
from abc import ABC, abstractmethod
from tooldistillator import summarize

logging.basicConfig(
    format="%(asctime)s: %(filename)s: %(message)s",
    datefmt="%m/%d/%Y %I:%M:%S %p",
    level=logging.INFO,
    force=True,
)
logger = logging.getLogger("tooldistillator")


class ToolDistillatorResultIterator(ABC):
    """
    Base class for the parsers for each tool

    This should return an appropriate iterator for results
    from whatever
    tool report is being parsed
    """

    def __init__(self, source, metadata):
        """
        Create an toolDistillatorResultIterator for which ever tool report is
        being parsed

        Based on:
        github.com/biopython/biopython/blob/master/Bio/SeqIO/Interfaces.py#L23

        Arguments:
            - source: input file stream or path to input
            - tool: name of tool report that is being parsed
        """
        self.source = source
        self.metadata = metadata
        self.report_name = f"{self.metadata['analysis_software_name']}_report"
        self._check_input_files(self.source)
        self._tooldistillator_structure()

    def _check_input_files(self, source, optional=False):
        """
        Check if the input is available to be parsed

        Check if the file exist, if it's empty or if path not exist

        Args:
            source (str): string of the file path
        """
        try:
            if os.stat(source).st_size > 0 and source:
                self.stream = open(source, "r")
                logger.info(f'Treatment of {source}')
            else:
                logger.error("Input file is empty")
                self.stream = False
        except (OSError, FileNotFoundError, IOError) as err:
            if optional:
                self.stream = False
            else:
                logger.error(f"{err}: input file not found")
                sys.exit()

    def extract_metadata_information(self, metadata_dic):
        extract_dic_info = copy.deepcopy(metadata_dic)
        for metadata_key, metadata_value in extract_dic_info.items():
            if "path" in metadata_key:
                hid_name = metadata_key.replace("path", "hid")
                if hid_name in metadata_dic.keys():
                    hid_value = metadata_dic[hid_name]
                else:
                    hid_value = None
                if metadata_value:
                    logger.info(f"Extraction of: {metadata_key}")
                    self._check_input_files(metadata_value)
                else:
                    logger.info(
                        f"No information for {metadata_key}: Not analyzed"
                        )

                self.tooldistillator_results["results"].append(
                    {
                        "name": metadata_key,
                        "file_path": metadata_value,
                        "hid": hid_value,
                        "galaxy_id": "",
                        "content": [],
                    }
                )
                self.tooldistillator_results.pop(metadata_key)
                self.tooldistillator_results.pop(hid_name)

    def _tooldistillator_structure(self):
        """
        Template structure for the final JSON file

        This template built with the tool name, then with
        a results dictionnary which contain some other dict
        to each parsed file
        Always keep tool name and path of parsed file
        """

        self.tooldistillator_results = {}
        for metadata_info in self.metadata:
            if self.metadata[metadata_info]:
                self.tooldistillator_results[metadata_info] = self.metadata[
                    metadata_info
                    ]
            else:
                self.tooldistillator_results[metadata_info] = ""
                if metadata_info == "tool_has_database":
                    self.tooldistillator_results[metadata_info] = False
            if "hid" not in self.metadata:
                self.tooldistillator_results["hid"] = ""

        self.tooldistillator_results["results"] = [
            {
                "name": self.report_name,
                "file_path": self.source,
                "hid": self.tooldistillator_results.pop("hid"),
                "galaxy_id": "",
                "content": [],
            }
        ]

    def _clean_dict_output(self, input_dict):
        """change non accepted error value by a null value
        Args:
            input_dict (dict): a python dict to parse
        """
        dict_str = json.dumps(input_dict, allow_nan=True)
        clean_dict = json.loads(dict_str, object_pairs_hook=self._dict_clean)
        return clean_dict

    def _dict_clean(self, items, replace_value_by=None):
        """replace all items from the na_value list
            by a null value accepted in json format
        Args:
            items (item): a dict item
            replace_value_by: value used to replace, default is None
        Returns:
            dict: sanitized dict to json export
        """
        na_values = [
            "#N/A",
            "#N/A N/A",
            "#NA",
            "-1.#IND",
            "-1.#QNAN",
            "-NaN",
            "-nan",
            "1.#IND",
            "1.#QNAN",
            "<NA>",
            "N/A",
            "NA",
            "NULL",
            "NaN",
            "None",
            "n/a",
            "nan",
            "null",
            None,
            "",
        ]
        result = {}
        for key, value in items:
            if str(value) in na_values:
                value = replace_value_by
            result[key] = value
        return result

    @abstractmethod
    def parse(self, handle):
        """
        Start parsing the file and return an tooldistillator iterator
        """

    def write(
        self,
        report_list,
        total_report_count=1,
        output_location=None,
    ):
        """
        Method to write to output the tooldistillator report
        (to either stdout or a filehandle) in json format

        Get number of reports and which report this one is
        """

        if output_location:
            # appending if more than one report
            if os.path.exists(output_location) and total_report_count > 1:
                out_fh = open(output_location, "a")
            else:
                out_fh = open(output_location, "w")
        else:
            out_fh = sys.stdout

        # remove empty value from dicts
        """report_list = [
            self._filter_json_output(report) for report in report_list
        ]"""
        report_list = self._clean_dict_output(report_list)
        json.dump(report_list, out_fh)

        if out_fh is not sys.stdout:
            out_fh.close()


def generate_tool_subparser(subparser, analysis_tool):
    """
    Build the argument parser for a specific tool

    (used to generate a tool-specific cli-parser and a generic tool parser)
    """
    report_file = tooldistillator._ReportFileToUse[
        analysis_tool
        ]
    description = (
        "Extract information from output(s) of "
        f"{analysis_tool} ({report_file})"
    )
    usage = f"tooldistillator.py {analysis_tool} <options>"
    help = (
        "Extract information from "
        f"{analysis_tool}'s output report i.e., {report_file}"
    )

    tool_parser = subparser.add_parser(
        analysis_tool,
        description=description,
        usage=usage,
        help=help,
    )

    tool_parser.add_argument(
        "report",
        nargs="+",
        help="Path to report(s)",
    )
    tool_parser.add_argument(
        "-o",
        "--output",
        default=None,
        help="Output location",
    )

    # any missing mandatory fields need supplied as CLI argument
    required_mandatory_metadata = \
        tooldistillator._RequiredToolMetadata[
            analysis_tool
        ]

    for field in required_mandatory_metadata:
        tool_parser.add_argument(
            f"--{field}",
            required=False,
            help=f"{required_mandatory_metadata[field]} for {analysis_tool}",
        )

    return subparser


def generate_summarize_subparser(subparser):
    """Add summary argument to the subparser

    Args:
        subparser (argparse._SubParsersAction):
        Subparser with provied tools arguments

    Returns:
        argparse._SubParsersAction: Complete parser
    """

    # add summarize subparser
    # not very pretty to have this tied into the analysis tools list
    # but there still doesn't seem a good way to group subparsers in
    # the argparse library
    description = "Aggregate several reports"
    usage = "tooldistillator summarize <options> <list of reports>"
    summarize_help = (
        "Aggregate several JSON reports generated "
        "from a tool output"
    )

    summarize_subparser = subparser.add_parser(
        "summarize",
        description=description,
        usage=usage,
        help=summarize_help,
    )

    summarize_subparser.add_argument(
        "-o",
        "--output",
        type=str,
        default=None,
        help="Output file path for summary",
    )

    summarize_subparser.add_argument(
        "-f",
        "--force",
        action="store_true",
        help="Overwrite to the output file mandatory",
    )
    summarize_subparser.add_argument(
        "tooldistillator_reports",
        nargs="+",
        help="list of tooldistillator reports",
    )
    return summarize_subparser


def generic_cli_interface():
    """
    Generate a generic tool report parser that passes to the tool specific
    parser
    """
    parser = argparse.ArgumentParser(
        description=(
            "Extract information from "
            "tool output(s) to JSON and/or"
            "aggregate several JSON reports"
        ),
        prog="tooldistillator",
        usage="tooldistillator <tool> <options>",
    )

    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"%(prog)s {tooldistillator.__version__}",
    )
    parser.add_argument(
        "--logfile",
        default=None,
        help="Log file location",
    )

    # add tool specific parsers
    subparser = parser.add_subparsers(
        title="Tools",
        help="",
        dest="analysis_tool",
    )

    for tool in tooldistillator._RequiredToolMetadata.keys():
        subparser = generate_tool_subparser(subparser, tool)

    generate_summarize_subparser(subparser)

    args = parser.parse_args()
    if args.logfile:
        print(args.logfile)
        logging.basicConfig(filename=args.logfile, filemode="w", force=True)

    if args.analysis_tool and args.analysis_tool != "summarize":
        required_mandatory_metadata = (
            tooldistillator._RequiredToolMetadata[
                args.analysis_tool
                ]
        )
        metadata = {f: getattr(args, f) for f in required_mandatory_metadata}
        # parse reports and write as appropriate (only first report with head
        # in tsv mode)
        # check number of reports and append correctly if >1
        total_report_count = len(args.report)
        report_list = []
        for report in args.report:
            parsed_report = tooldistillator.parse(
                report,
                metadata,
                args.analysis_tool,
            )
            report_list.append(parsed_report.tooldistillator_results)

        parsed_report.write(
            report_list=report_list,
            total_report_count=total_report_count,
            output_location=args.output,
        )

    elif args.analysis_tool == "summarize":
        export = summarize.ReportSummary(
            report_list=args.tooldistillator_reports,
            output_location=args.output,
            overwrite=args.force,
        )
        export.export_summary()
    else:
        parser.print_help()
