#!/usr/bin/env python
import json
import pandas as pd
from tooldistillator.interfaces import (
    ToolDistillatorResultIterator,
    logger
)

required_metadata = {
    "hid": "historic ID to bakta result file from galaxy",
    "analysis_software_version": "bakta version",
    "reference_database_version": "DB version",
    "annotation_tabular_path": "annotation file in TSV format",
    "annotation_tabular_hid":
        "historic ID for annotation tsv file from galaxy",
    "gff_file_path": "annotation file result in gff3 format",
    "gff_file_hid": "historic ID for gff file from galaxy",
    "annotation_genbank_path": "annotation file in genbank format",
    "annotation_genbank_hid":
        "historic ID for annotation genbank file from galaxy",
    "annotation_embl_path": "annotation file in embl format",
    "annotation_embl_hid": "historic ID for annotation embl file from galaxy",
    "contig_sequences_path": "contig sequences in fasta ([output].fna)",
    "contig_sequences_hid": "historic ID for contigs fasta file from galaxy",
    "nucleotide_annotation_path":
        "nuleotide file ([output].ffn) of the annotation",
    "nucleotide_annotation_hid": "historic ID for nucleotide file from galaxy",
    "amino_acid_annotation_path": "amino acid file of the annotation",
    "amino_acid_annotation_hid":
        "historic ID for amino acide sequence file from galaxy",
    "hypothetical_protein_path":
        "hypothetical protein CDS amino acid sequences as fasta",
    "hypothetical_protein_hid":
        "historic ID for hypothetical protein file file from galaxy",
    "hypothetical_tabular_path": "hypothetical protein CDS",
    "hypothetical_tabular_hid":
    "historic ID for hypothetical tabular file from galaxy",
    "summary_result_path": "summary file of the bakta analysis in txt format",
    "summary_result_hid": "historic ID for summary file from galaxy",
    "plot_file_path": "genome annotation plot file path",
    "plot_file_hid": "historic ID for plot file from galaxy",
}


class BaktaIterator(ToolDistillatorResultIterator):
    """Bakta tool parser
    Filter results from bakta json output file
    Could also filter file for:
    summary file
    gff annotation file
    nucleotide sequence file
    amino acid sequence file

    Args:
        ToolDistillatorResultIterator: abstract class to the parser
    """

    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "bakta"
        self.metadata = metadata
        super().__init__(source, self.metadata)
        self.tooldistillator_results["tool_has_database"] = True
        self.parse()

    def parse(self):
        """Extract information from bakta json file
        Add some optionnal informations if provided
        Add summary result with global result
        Add path for gff3 file
        Add path for nucleotide sequences
        Add path for amino acid sequences
        """
        # skip any manually specified fields for later
        if self.stream:
            with open(self.stream.name, "r") as f:
                bakta_load = json.loads(f.read())
            logger.info("bakta input file imported")
            bakta_load.pop("sequences")
            features_df = pd.DataFrame.from_records(bakta_load["features"])
            features_df = features_df.drop(["nt", "aa"], axis=1)
            features_json = features_df.to_json(orient="records")
            bakta_load["features"] = json.loads(features_json)
            if self.tooldistillator_results["analysis_software_version"] == "":
                self.tooldistillator_results[
                    "analysis_software_version"
                    ] = bakta_load[
                    "version"
                ].pop("bakta")
            v = self.tooldistillator_results["reference_database_version"]
            if v == "":
                self.tooldistillator_results[
                    "reference_database_version"
                    ] = bakta_load[
                    "version"
                ].pop("db")
            bakta_load.pop("version")
            # extract summary file
            if self.metadata["summary_result_path"]:
                logger.info("summary_result_path analysis")
                self._check_input_files(self.metadata["summary_result_path"])
                summary_df = pd.read_table(self.stream, header=7)
                summary_df = summary_df["Annotation:"].str.replace(" ", "")
                summary_df = summary_df.str.split(
                    pat=":", n=1, expand=True)
                # Indexes
                start_row = 0
                end_row = summary_df[summary_df[0] == "Bakta"].index[0]
                start_col = 0
                end_col = 2
                summary_df = summary_df.iloc[
                    start_row:end_row,
                    start_col:end_col,
                ]
                summary_df[1] = summary_df[1].astype("int")
                sequence_dic = dict(zip(summary_df[0], summary_df[1]))
                bakta_load["summary"] = sequence_dic
            self.tooldistillator_results["results"][0]["content"].append(
                bakta_load
            )
        self.extract_metadata_information(self.metadata)
