#!/usr/bin/env python
import json
import pandas as pd
from tooldistillator.interfaces import (
    ToolDistillatorResultIterator,
    logger
)

required_metadata = {
    "hid": ("historic ID to recentrifuge data file provided by Galaxy"),
    "analysis_software_version": "recentrifuge version",
    "reference_database_version": "ncbi taxonomy DB version",
    "analysis_software_version": "recentrifuge version",
    "rcf_stat_path": "recentrifuge statistic file",
    "rcf_stat_hid": "historic ID provided by Galaxy",
    "rcf_html_path": "recentrifuge html report file",
    "rcf_html_hid": "recentrifuge html report file",
}


class RecentrifugeIterator(ToolDistillatorResultIterator):
    """Recentrifuge tool parser
    parse result from tabular data
    Could also parse stat file
    Could keep path information to html report

    Args:
        ToolDistillatorResultIterator: abstract class to the parser
    """

    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "recentrifuge"
        self.metadata = metadata
        self.rcf_stat_colnames = [
            "infos",
            "value",
        ]
        super().__init__(source, self.metadata)
        self.tooldistillator_results["tool_has_database"] = True
        self.parse()

    def parse(self):
        """Parser for recentrifuge
        Could parse data and stat file
        Extract only html report path
        """
        # skip any manually specified fields for later
        if self.stream:
            reader = self._filter_dataframe_type(
                self.stream, header_position=1
                )
            self.tooldistillator_results["results"][0]["content"] = reader

        {
            self._get_results(optional_file)
            for optional_file in self.metadata.keys()
            if 'path' in optional_file
        }

    def _filter_dataframe_type(
        self,
        handle,
        header_position=0,
        separator="\t",
        orient_type="records",
        col_names=None,
        skip=None,
    ):
        """Clean input recentrifuge files
        Remove bad character and keep only value informations

        Args:
            handle (_io.TextIOWrapper): opened file
            header_position (int, optional): header to use as column name.
            Defaults 0.
            separator (str, optional): Separator type. Defaults to "\t".
            orient_type (str, optional): Store dict format column and value.
            Defaults to "records".
            col_names (list, optional): List of name to replace colnames.
            Defaults to None.
            skip (int, optional): Row number to not use. Defaults to None.

        Returns:
            dict: filtered dictionnary
        """
        try:
            reader = pd.read_table(
                handle,
                header=header_position,
                sep=separator,
                skiprows=skip,
            )
            if not reader.empty:
                if col_names:
                    reader.columns = col_names
                reader = reader.replace("^[ \t]+", "", regex=True)
                reader = reader.replace("[.]$", "", regex=True)
                reader = reader.replace(" ", "", regex=True)
                reader.columns = reader.columns.str.replace("%", "")
                reader = json.loads(reader.to_json(orient=orient_type))
                return reader
        except ValueError as ve:
            logger.error(f"{ve}, bad input file for {self.stream}")

    def _get_results(self, filetype: str):
        """extract results to each dataframe and
        add output to the final abromics result list
        Args:
            filetype (str): type of input file
            need to filter dataframe
        """
        file_path = self.metadata.get(filetype)
        self._check_input_files(file_path, optional=True)
        if self.stream and "html" not in filetype:
            reader = self._filter_dataframe_type(
                self.stream,
                header_position=None,
                col_names=self.rcf_stat_colnames,
                skip=1
                )
        else:
            logger.info(f'No file associated to {filetype}')
            reader = []
        self._fill_tooldistillator_results(reader=reader, filetype=filetype)

    def _fill_tooldistillator_results(self, reader, filetype: str):
        """Fill the final abromics object with results

        Args:
            filetype (str): _description_
            reader (pandas.core.frame.DataFrame): trimmed pandas dataframe
        """
        hid_info = filetype.replace("path", "hid")
        logger.info(filetype)
        report_name = filetype.replace("_path", "")
        try:
            self.tooldistillator_results["results"].append(
                {
                    "name": report_name,
                    "file_path": self.tooldistillator_results.pop(
                        filetype),
                    "hid": self.tooldistillator_results.pop(
                        hid_info
                        ),
                    "galaxy_id": "",
                    "content": reader,
                }
            )
        except KeyError:
            logger.error(f'{KeyError}, {filetype} is not in item list')
            raise KeyError(f'{filetype} is not in item list')
